@@ -61,11 +61,6 @@ class JsRoutes
     end
 
     def generate(opts = {})
-      # Ensure routes are loaded. If they're not, load them.
-      if Rails.application.routes.named_routes.to_a.empty?
-        Rails.application.reload_routes!
-      end
-
       new(opts).generate
     end
 
@@ -101,18 +96,24 @@ class JsRoutes
   end
 
   def generate
+    # Ensure routes are loaded. If they're not, load them.
+    if named_routes.to_a.empty? && application.respond_to?(:reload_routes!)
+      application.reload_routes!
+    end
+
     {
       "GEM_VERSION"         => JsRoutes::VERSION,
-      "APP_CLASS"           => Rails.application.class.to_s,
+      "APP_CLASS"           => application.class.to_s,
       "NAMESPACE"           => @options[:namespace],
       "DEFAULT_URL_OPTIONS" => json(@options[:default_url_options].merge(deprecate_url_options)),
       "PREFIX"              => @options[:prefix] || Rails.application.config.relative_url_root || "",
       "NODE_TYPES"          => json(NODE_TYPES),
       "SERIALIZER"          => @options[:serializer] || json(nil),
       "ROUTES"              => js_routes,
-      "SPECIAL_OPTIONS_KEY" => @options[:special_options_key].to_s
+      "SPECIAL_OPTIONS_KEY" => @options[:special_options_key].to_s,
+      "DEPRECATED_BEHAVIOR" => Rails.version < "4",
     }.inject(File.read(File.dirname(__FILE__) + "/routes.js")) do |js, (key, value)|
-      js.gsub!(key, value)
+      js.gsub!(key, value.to_s)
     end
   end
 
@@ -152,8 +153,16 @@ class JsRoutes
 
   protected
 
+  def application
+    @options[:application] || Rails.application
+  end
+
+  def named_routes
+    application.routes.named_routes.to_a
+  end
+
   def js_routes
-    js_routes = Rails.application.routes.named_routes.to_a.sort_by(&:first).flat_map do |_, route|
+    js_routes = named_routes.sort_by(&:first).flat_map do |_, route|
       [build_route_if_match(route)] + mounted_app_routes(route)
     end.compact
     "{\n" + js_routes.join(",\n") + "}\n"
@@ -208,13 +217,19 @@ class JsRoutes
   end
 
   def route_js_arguments(route, parent_spec)
-    required_parts = route.required_parts.clone
-    optional_parts = route.parts - required_parts
-    default_parts = route.defaults.select do |part, _|
-      FILTERED_DEFAULT_PARTS.exclude?(part) && URL_OPTIONS.include?(part) || required_parts.include?(part)
+    required_parts = route.required_parts
+    parts_table = route.parts.each_with_object({}) do |part, hash|
+      hash[part] = required_parts.include?(part)
+    end
+    default_options = route.defaults.select do |part, _|
+      FILTERED_DEFAULT_PARTS.exclude?(part) && URL_OPTIONS.include?(part) || parts_table[part]
     end
     [
-      required_parts, optional_parts, serialize(route.path.spec, parent_spec), default_parts
+      # JS objects don't preserve the order of properties which is crucial,
+      # so array is a better choice.
+      parts_table.to_a,
+      default_options,
+      serialize(route.path.spec, parent_spec)
     ].map do |argument|
       json(argument)
     end.join(", ")
