@@ -14,6 +14,17 @@ describe JsRoutes, "compatibility with Rails"  do
     expect(evaljs("Routes.inbox_path(1)")).to eq(routes.inbox_path(1))
   end
 
+  it "should raise error if required argument is not passed" do
+    expect { evaljs("Routes.thing_path()") }
+      .to raise_error('Route parameter missing: id')
+    expect { evaljs("Routes.search_path()") }
+      .to raise_error('Route parameter missing: q')
+    expect { evaljs("Routes.book_path()") }
+      .to raise_error('Route parameter missing: title')
+    expect { evaljs("Routes.book_title_path()") }
+      .to raise_error('Route parameter missing: title')
+  end
+
   it "should support 0 as a member parameter" do
     expect(evaljs("Routes.inbox_path(0)")).to eq(routes.inbox_path(0))
   end
@@ -59,9 +70,6 @@ describe JsRoutes, "compatibility with Rails"  do
     end
 
     it 'should support route default subdomain' do
-      # root inside namespace is broken
-      # https://github.com/rails/rails/pull/23235
-      pending if Rails.version == '5.0.0'
       expect(evaljs("Routes.backend_root_path()")).to eq(routes.backend_root_path)
     end
 
@@ -69,6 +77,10 @@ describe JsRoutes, "compatibility with Rails"  do
       expect(evaljs("Routes.api_purchases_path({format: 'xml'})")).to eq(routes.api_purchases_path(format: 'xml'))
     end
 
+    it "should support default format override by passing it in args" do
+      expect(evaljs("Routes.api_purchases_path('xml')")).to eq(routes.api_purchases_path('xml'))
+    end
+
     it "doesn't apply defaults to path" do
       expect(evaljs("Routes.with_defaults_path()")).to eq(routes.with_defaults_path)
     end
@@ -184,11 +196,10 @@ describe JsRoutes, "compatibility with Rails"  do
     context "but not including them" do
       it "should not include the optional parts" do
         expect(evaljs("Routes.things_path()")).to eq(routes.things_path)
+        expect(evaljs("Routes.things_path({ q: 'hello' })")).to eq(routes.things_path(q: 'hello'))
       end
 
       it "should not require the optional parts as arguments" do
-        #TODO: fix this inconsistence
-        pending
         expect(evaljs("Routes.thing_path(null, 5)")).to eq(routes.thing_path(nil, 5))
       end
 
@@ -196,19 +207,68 @@ describe JsRoutes, "compatibility with Rails"  do
         expect(evaljs("Routes.thing_path(5, {optional_id: undefined})")).to eq(routes.thing_path(5, :optional_id => nil))
       end
 
+      it "should raise error when passing non-full list of arguments and some query params" do
+        expect { evaljs("Routes.thing_path(5, {q: 'hello'})") }
+          .to raise_error('Route parameter missing: id')
+      end
+
       it "should treat null as non-given optional part" do
         expect(evaljs("Routes.thing_path(5, {optional_id: null})")).to eq(routes.thing_path(5, :optional_id => nil))
       end
+
+      it "should work when passing required params in options" do
+        expect(evaljs("Routes.thing_deep_path({second_required: 1, third_required: 2})")).to eq(routes.thing_deep_path(second_required: 1, third_required: 2))
+      end
+
+      it "should skip leading and trailing optional parts" do
+        skip if Rails.version < '4'
+        expect(evaljs("Routes.thing_deep_path(1, 2)")).to eq(routes.thing_deep_path(1, 2))
+      end
     end
 
     context "and including them" do
+      if Rails.version < '4'
+        it "should fail when insufficient arguments are given" do
+          expect { evaljs("Routes.thing_deep_path(1)")}
+            .to raise_error('Route parameter missing: second_required')
+          expect { evaljs("Routes.thing_deep_path(1,2)")}
+            .to raise_error('Route parameter missing: third_required')
+        end
+      end
+
       it "should include the optional parts" do
         expect(evaljs("Routes.things_path({optional_id: 5})")).to eq(routes.things_path(:optional_id => 5))
+        expect(evaljs("Routes.things_path(5)")).to eq(routes.things_path(5))
+        expect(evaljs("Routes.thing_deep_path(1, { third_required: 3, second_required: 2 })")).to eq(routes.thing_deep_path(1, third_required: 3, second_required: 2))
+        expect(evaljs("Routes.thing_deep_path(1, { third_required: 3, second_required: 2, forth_optional: 4 })")).to eq(routes.thing_deep_path(1, third_required: 3, second_required: 2, forth_optional: 4))
+        expect(evaljs("Routes.thing_deep_path(2, { third_required: 3, first_optional: 1 })")).to eq(routes.thing_deep_path(2, third_required: 3, first_optional: 1))
+        expect(evaljs("Routes.thing_deep_path(3, { first_optional: 1, second_required: 2 })")).to eq(routes.thing_deep_path(3, first_optional: 1, second_required: 2))
+        expect(evaljs("Routes.thing_deep_path(3, { first_optional: 1, second_required: 2, forth_optional: 4 })")).to eq(routes.thing_deep_path(3, first_optional: 1, second_required: 2, forth_optional: 4))
+        expect(evaljs("Routes.thing_deep_path(4, { first_optional: 1, second_required: 2, third_required: 3 })")).to eq(routes.thing_deep_path(4, first_optional: 1, second_required: 2, third_required: 3))
+        expect(evaljs("Routes.thing_deep_path(1, 2, { third_required: 3 })")).to eq(routes.thing_deep_path(1, 2, third_required: 3))
+        expect(evaljs("Routes.thing_deep_path(1,2, {third_required: 3, q: 'bogdan'})")).to eq(routes.thing_deep_path(1,2, {third_required: 3, q: 'bogdan'}))
+        expect(evaljs("Routes.thing_deep_path(1, 2, { forth_optional: 4, third_required: 3 })")).to eq(routes.thing_deep_path(1, 2, forth_optional: 4, third_required: 3))
+        expect(evaljs("Routes.thing_deep_path(1, 3, { second_required: 2 })")).to eq(routes.thing_deep_path(1, 3, second_required: 2))
+        expect(evaljs("Routes.thing_deep_path(1, 4, { second_required: 2, third_required: 3 })")).to eq(routes.thing_deep_path(1, 4, second_required: 2, third_required: 3))
+        expect(evaljs("Routes.thing_deep_path(2, 3, { first_optional: 1 })")).to eq(routes.thing_deep_path(2, 3, first_optional: 1))
+        expect(evaljs("Routes.thing_deep_path(2, 3, { first_optional: 1, forth_optional: 4 })")).to eq(routes.thing_deep_path(2, 3, first_optional: 1, forth_optional: 4))
+        expect(evaljs("Routes.thing_deep_path(2, 4, { first_optional: 1, third_required: 3 })")).to eq(routes.thing_deep_path(2, 4, first_optional: 1, third_required: 3))
+        expect(evaljs("Routes.thing_deep_path(3, 4, { first_optional: 1, second_required: 2 })")).to eq(routes.thing_deep_path(3, 4, first_optional: 1, second_required: 2))
+        expect(evaljs("Routes.thing_deep_path(1, 2, 3)")).to eq(routes.thing_deep_path(1, 2, 3))
+        expect(evaljs("Routes.thing_deep_path(1, 2, 3, { forth_optional: 4 })")).to eq(routes.thing_deep_path(1, 2, 3, forth_optional: 4))
+        expect(evaljs("Routes.thing_deep_path(1, 2, 4, { third_required: 3 })")).to eq(routes.thing_deep_path(1, 2, 4, third_required: 3))
+        expect(evaljs("Routes.thing_deep_path(1, 3, 4, { second_required: 2 })")).to eq(routes.thing_deep_path(1, 3, 4, second_required: 2))
+        expect(evaljs("Routes.thing_deep_path(2, 3, 4, { first_optional: 1 })")).to eq(routes.thing_deep_path(2, 3, 4, first_optional: 1))
+        expect(evaljs("Routes.thing_deep_path(1, 2, 3, 4)")).to eq(routes.thing_deep_path(1, 2, 3, 4))
+
       end
 
       context "on nested optional parts" do
-        it "should include everything that is not optional" do
-          expect(evaljs("Routes.classic_path({controller: 'classic', action: 'edit'})")).to eq(routes.classic_path(controller: :classic, action: :edit))
+        if Rails.version <= "5.0.0"
+          # this type of routing is deprecated
+          it "should include everything that is not optional" do
+            expect(evaljs("Routes.classic_path({controller: 'classic', action: 'edit'})")).to eq(routes.classic_path(controller: :classic, action: :edit))
+          end
         end
       end
     end
@@ -229,7 +289,19 @@ describe JsRoutes, "compatibility with Rails"  do
 
     it "should throw Exceptions if when there is too many parameters" do
       expect {
-        evaljs("Routes.inbox_path(1,2)")
+        evaljs("Routes.inbox_path(1,2,3)")
+      }.to raise_error(js_error_class)
+    end
+
+    it "should throw Exceptions if when pass id with null" do
+      expect {
+        evaljs("Routes.inbox_path({id: null})")
+      }.to raise_error(js_error_class)
+    end
+
+    it "should throw Exceptions if when pass to_param with null" do
+      expect {
+        evaljs("Routes.inbox_path({to_param: null})")
       }.to raise_error(js_error_class)
     end
   end
